@@ -1,8 +1,8 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##	$Id: readmail.pl,v 2.21 2002/10/15 22:06:53 ehood Exp $
+##	$Id: readmail.pl,v 2.35 2004/03/10 22:01:33 ehood Exp $
 ##  Author:
-##      Earl Hood       mhonarc@mhonarc.org
+##      Earl Hood       mhonarc AT mhonarc DOT org
 ##  Description:
 ##      Library defining routines to parse MIME e-mail messages.  The
 ##	library is designed so it may be reused for other e-mail
@@ -17,13 +17,14 @@
 ##	$hash_ref 	= MAILread_file_header($handle);
 ##	$hash_ref 	= MAILread_header($mesg_str_ref);
 ##
-##	($disp, $file)  = MAILhead_get_disposition($fields_hash_ref);
+##	($disp, $file, $raw, $html_name)  =
+##			  MAILhead_get_disposition($fields_hash_ref, $do_html);
 ##	$boolean 	= MAILis_excluded($content_type);
 ##	$parm_hash_ref  = MAILparse_parameter_str($header_field);
 ##	$parm_hash_ref  = MAILparse_parameter_str($header_field, 1);
 ##
 ##---------------------------------------------------------------------------##
-##    Copyright (C) 1996-2001	Earl Hood, mhonarc@mhonarc.org
+##    Copyright (C) 1996-2003	Earl Hood, mhonarc AT mhonarc DOT org
 ##
 ##    This program is free software; you can redistribute it and/or modify
 ##    it under the terms of the GNU General Public License as published by
@@ -47,8 +48,7 @@ package readmail;
 ##	Private Globals							     ##
 ###############################################################################
 
-my $Url	          = '(\w+://|\w+:)';
-
+#my $Url	  = '(\w+://|\w+:)';
 my @_MIMEAltPrefs = ();
 my %_MIMEAltPrefs = ();
 
@@ -60,9 +60,18 @@ my %_MIMEAltPrefs = ();
 ##	Constants
 ##
 
+## String for matching the start of a URL
+$UrlRxStr	  =
+    '(?:(?:https?|ftp|afs|wais|telnet|ldap|gopher|z39\.50[rs]|vemmi|imap|'.
+	  'nfs|acap|rtspu?|tip|pop|sip|(?:soap|xmlrpc)\.beeps?|go|ipp|'.
+	  'tftp)://|'.
+       'news:(?://)?|'.
+       '(?:nntp|mid|cid|mailto|prospero|data|service|tel|fax|modem|h\.323):)';
+
 ##  Constants for use as second argument to MAILdecode_1522_str().
 sub JUST_DECODE() { 1; }
 sub DECODE_ALL()  { 2; }
+sub TEXT_ENCODE() { 3; }
 
 ##---------------------------------------------------------------------------##
 
@@ -106,35 +115,6 @@ $DecodeHeader	= 0;
 %MIMEDecodersSrc		= ()
     unless defined(%MIMEDecodersSrc);
 
-##	Default settings:
-$MIMEDecoders{"7bit"}		  = "as-is"
-    unless defined($MIMEDecoders{"7bit"});
-$MIMEDecoders{"8bit"}		  = "as-is"
-    unless defined($MIMEDecoders{"8bit"});
-$MIMEDecoders{"binary"}		  = "as-is"
-    unless defined($MIMEDecoders{"binary"});
-$MIMEDecoders{"base64"}		  = "base64::b64decode"
-    unless defined($MIMEDecoders{"base64"});
-$MIMEDecoders{"quoted-printable"} = "quoted_printable::qprdecode"
-    unless defined($MIMEDecoders{"quoted-printable"});
-$MIMEDecoders{"x-uuencode"}	  = "base64::uudecode"
-    unless defined($MIMEDecoders{"x-uuencode"});
-$MIMEDecoders{"x-uue"}     	  = "base64::uudecode"
-    unless defined($MIMEDecoders{"x-uue"});
-$MIMEDecoders{"uuencode"}  	  = "base64::uudecode"
-    unless defined($MIMEDecoders{"uuencode"});
-
-$MIMEDecodersSrc{"base64"}	  	= "base64.pl"
-    unless defined($MIMEDecodersSrc{"base64"});
-$MIMEDecodersSrc{"quoted-printable"}	= "qprint.pl"
-    unless defined($MIMEDecodersSrc{"quoted-printable"});
-$MIMEDecodersSrc{"x-uuencode"}	 	= "base64.pl"
-    unless defined($MIMEDecodersSrc{"x-uuencode"});
-$MIMEDecodersSrc{"x-uue"}     	 	= "base64.pl"
-    unless defined($MIMEDecodersSrc{"x-uue"});
-$MIMEDecodersSrc{"uuencode"}  	 	= "base64.pl"
-    unless defined($MIMEDecodersSrc{"uuencode"});
-
 ## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ##  %MIMECharSetConverters is the associative array for storing functions
 ##  for converting data in a particular charset to a destination format
@@ -171,10 +151,6 @@ $MIMEDecodersSrc{"uuencode"}  	 	= "base64.pl"
 %MIMECharSetConvertersSrc		= ()
     unless defined(%MIMECharSetConvertersSrc);
 
-##	Default settings:
-$MIMECharSetConverters{"default"}	= "-ignore-"
-    unless defined($MIMECharSetConverters{"default"});
-
 ## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ##  %MIMEFilters is the associative array for storing functions that
 ##  process various content-types in the MAILread_body routine.
@@ -240,6 +216,28 @@ $MIMECharSetConverters{"default"}	= "-ignore-"
 %MIMECharsetAliases = ()
     unless defined(%MIMECharsetAliases);
 
+##---------------------------------------------------------------------------
+##	Text entity-related variables
+##
+
+##  Default character set if none specified.
+$TextDefCharset = 'us-ascii'
+    unless defined($TextDefCharset);
+
+##  Destination character encoding for text entities.
+$TextEncode = undef
+    unless defined($TextEncode);
+##  Text encoding function.
+$TextEncoderFunc = undef
+    unless defined($TextEncodingFunc);
+##  Text encoding function source file.
+$TextEncoderSrc = undef
+    unless defined($TextEncodingSrc);
+
+##  Prefilter function
+$TextPreFilter  = undef
+    unless defined($TextPreFilter);
+
 ##---------------------------------------------------------------------------
 ##	Variables holding functions for generating processed output
 ##	for MAILread_body().  The default functions generate HTML.
@@ -310,17 +308,22 @@ $FormatHeaderFunc		= undef
 ##
 ##	Usage:
 ##
-##	    $ret_data = &MAILdecode_1522_str($str, $decoding_flag);
+##	    $ret_data = &MAILdecode_1522_str($str, $dec_flag);
 ##
-##	If $decoding_flag is JUST_DECODE, $str will be decoded for only
+##	If $dec_flag is JUST_DECODE, $str will be decoded for only
 ##	the charsets specified as "-decode-".  If it is equal to
 ##	DECODE_ALL, all encoded data is decoded without any conversion.
+##	If $dec_flag is TEXT_ENCODE, then all data will be converted
+##	and encoded according to $readmail::TextEncode and
+##	$readmail::TextEncoderFunc.
 ##
 sub MAILdecode_1522_str {
-    my($str) = shift;
-    my($decoding_flag) = shift || 0;
+    my $str      = shift;
+    my $dec_flag = shift || 0;
+    my $ret      = ('');
     my($charset,
        $encoding,
+       $pos,
        $dec,
        $charcnv,
        $real_charset,
@@ -328,64 +331,96 @@ sub MAILdecode_1522_str {
        $plain_real_charset,
        $strtxt,
        $str_before);
-    my($ret) = ('');
+
+    # Get text encoder
+    my $encfunc  = undef;
+    if ($dec_flag == TEXT_ENCODE) {
+	$encfunc = load_textencoder();
+	if (!defined($encfunc)) {
+	    $encfunc = undef  unless defined($encfunc);
+	    $dec_flag = 0;
+	}
+    }
 
     # Get plain converter
     ($plaincnv, $plain_real_charset) = MAILload_charset_converter('plain');
     $plain_real_charset = 'us-ascii'  if $plain_real_charset eq 'plain';
 
     # Decode string
-    while ($str =~ /=\?([^?]+)\?(.)\?([^?]*)\?=/) {
-
+    my $firsttime = 1;
+    while ($str =~ /(=\?([^?]+)\?(.)\?([^?]*)\?=)/g) {
 	# Grab components
-	($charset, $encoding) = ($1, $2);
-	$strtxt = $3; $str_before = $`; $str = $';
+	$pos = pos($str);
+	($charset, $encoding, $strtxt) = (lc($2), lc($3), $4);
+	$str_before = substr($str, 0, $pos-length($1));
+	substr($str, 0, $pos) = '';
+	pos($str) = 0;
 
 	# Check encoding method and grab proper decoder
-	if ($encoding =~ /b/i) {
+	if ($encoding eq 'b') {
 	    $dec = &load_decoder('base64');
 	} else {
 	    $dec = &load_decoder('quoted-printable');
 	}
 
 	# Convert before (unencoded) text
-	if ($decoding_flag) {				# ignore if just decode
-	    $ret .= $str_before;
-	} elsif (defined(&$plaincnv)) {			# decode and convert
-	    $ret .= &$plaincnv($str_before, $plain_real_charset);
-	} else {					# ignore
-	    $ret .= $str_before;
+	if ($firsttime || $str_before =~ /\S/) {
+	    if (defined($encfunc)) {			# encoding
+		&$encfunc(\$str_before, $plain_real_charset, $TextEncode);
+		$ret .= $str_before;
+	    } elsif ($dec_flag) {			# ignore if just decode
+		$ret .= $str_before;
+	    } elsif (defined(&$plaincnv)) {		# decode and convert
+		$ret .= &$plaincnv($str_before, $plain_real_charset);
+	    } else {					# ignore
+		$ret .= $str_before;
+	    }
 	}
+	$firsttime = 0;
 
-	# Convert encoded text
-	if ($decoding_flag == DECODE_ALL) {
-	    $charcnv = '-decode-';
-	} else {
-	    ($charcnv, $real_charset) = MAILload_charset_converter($charset);
-	}
-
-	# Decode only
-	if ($charcnv eq '-decode-') {
+	# Encoding text
+	if (defined($encfunc)) {
+	    $real_charset = $MIMECharsetAliases{$charset}
+			    ? $MIMECharsetAliases{$charset} : $charset;
 	    $strtxt =~ s/_/ /g;
-	    $ret .= &$dec($strtxt);
+	    $strtxt =  &$dec($strtxt);
+	    &$encfunc(\$strtxt, $real_charset, $TextEncode);
+	    $ret   .= $strtxt;
 
-	# Ignore if just decoding
-	} elsif ($decoding_flag) {
-	    $ret .= "=?$charset?$encoding?$strtxt?=";
-
-	# Decode and convert
-	} elsif (defined(&$charcnv)) {
-	    $strtxt =~ s/_/ /g;
-	    $ret .= &$charcnv(&$dec($strtxt), $real_charset);
-
-	# Fallback is to ignore
+	# Regular conversion
 	} else {
-	    $ret .= "=?$charset?$encoding?$strtxt?=";
+	    if ($dec_flag == DECODE_ALL) {
+		$charcnv = '-decode-';
+	    } else {
+		($charcnv, $real_charset) =
+		    MAILload_charset_converter($charset);
+	    }
+	    # Decode only
+	    if ($charcnv eq '-decode-') {
+		$strtxt =~ s/_/ /g;
+		$ret .= &$dec($strtxt);
+
+	    # Ignore if just decoding
+	    } elsif ($dec_flag) {
+		$ret .= "=?$charset?$encoding?$strtxt?=";
+
+	    # Decode and convert
+	    } elsif (defined(&$charcnv)) {
+		$strtxt =~ s/_/ /g;
+		$ret .= &$charcnv(&$dec($strtxt), $real_charset);
+
+	    # Fallback is to ignore
+	    } else {
+		$ret .= "=?$charset?$encoding?$strtxt?=";
+	    }
 	}
     }
 
     # Convert left-over unencoded text
-    if ($decoding_flag) {			# ignore if just decode
+    if (defined($encfunc)) {			# encoding
+	&$encfunc(\$str, $plain_real_charset, $TextEncode);
+	$ret .= $str;
+    } elsif ($dec_flag) {			# ignore if just decode
 	$ret .= $str;
     } elsif (defined(&$plaincnv)) {		# decode and convert
 	$ret .= &$plaincnv($str, $plain_real_charset);
@@ -458,9 +493,10 @@ sub MAILread_body {
     } else {
 	$type = $subtype = '';
     }
+    $fields->{'x-mha-content-type'} = $ctype;
 
     ## Check if type is excluded
-    if ($MIMEExcs{$ctype} || $MIMEExcs{$type}) {
+    if (MAILis_excluded($ctype)) {
 	return (&$ExcludedPartFunc($ctype));
     }
 
@@ -498,22 +534,35 @@ sub MAILread_body {
 	$encoding = undef;
 	$decodefunc = undef;
     }
+    my $decoded = 0;
+    if (defined($decodefunc) && defined(&$decodefunc)) {
+	$$body = &$decodefunc($$body);
+	$decoded = 1;
+    } elsif ($decodefunc =~ /as-is/i) {
+	$decoded = 1;
+    }
+
+    ## Convert text encoding
+    if ($type eq 'text') {
+	my $charset = extract_charset($content, $subtype, $body);
+	$fields->{'x-mha-charset'} = $charset;
+	my $textfunc = load_textencoder();
+	if (defined($textfunc)) {
+	    $fields->{'x-mha-charset'} = $TextEncode
+		if defined(&$textfunc($body, $charset, $TextEncode));
+	}
+	if (defined($TextPreFilter) && defined(&$TextPreFilter)) {
+	    &$TextPreFilter($fields, $body);
+	}
+    } else {
+	# define x-mha-charset in case text filter associated with
+	# a non-text type
+	$fields->{'x-mha-charset'} = $TextDefCharset;
+    }
 
     ## A filter is defined for given content-type
     if ($filter && defined(&$filter)) {
-	## decode data
-	if (defined($decodefunc)) {
-	    if (defined(&$decodefunc)) {
-		$decoded = &$decodefunc($$body);
-		@array = &$filter($fields, \$decoded, 1, $args);
-	    } else {
-		@array = &$filter($fields, $body,
-				  $decodefunc =~ /as-is/i, $args);
-	    }
-	} else {
-	    @array = &$filter($fields, $body, 0, $args);
-	}
-
+	@array = &$filter($fields, $body, $decoded, $args);
 	## Setup return variables
 	$ret = shift @array;				# Return string
 	push(@files, @array);				# Derived files
@@ -530,7 +579,7 @@ sub MAILread_body {
 	    if ($content =~ m/\bboundary\s*=\s*"([^"]*)"/i) {
 		$boundary = $1;
 	    } else {
-		($boundary) = $content =~ m/\bboundary\s*=\s*(\S+)/i;
+		($boundary) = $content =~ m/\bboundary\s*=\s*([^\s;]+)/i;
 		$boundary =~ s/;$//;  # chop ';' if grabbed
 	    }
 
@@ -570,16 +619,17 @@ sub MAILread_body {
 		    $$body =~ s/\A\r?\n//;
 		    $start_pos = 0;
 		}
-		if (!$have_end) {
-		    warn qq/Warning: No end boundary delimiter found in /,
-			 qq/message body\n/;
-		    push(@parts, $$body);
-		    $parts[$#parts] =~ s/^\r//;
-		    $$body = "";
-		}
 		if ($found) {
-		    # discard front-matter
-		    shift(@parts);
+		    if (!$have_end) {
+			warn qq/Warning: No end boundary delimiter found in /,
+			     qq/message body\n/;
+			push(@parts, $$body);
+			$parts[$#parts] =~ s/^\r//;
+			$$body = "";
+		    } else {
+			# discard front-matter
+			shift(@parts);
+		    }
 		} else {
 		    # no boundary separators in message!
 		    warn qq/Warning: No boundary delimiters found in /,
@@ -603,12 +653,14 @@ sub MAILread_body {
 	    my($cid, $href, $pctype);
 	    my %alt_exc = ( );
 	    my $have_alt_prefs = $isalt && scalar(@_MIMEAltPrefs);
+	    my $partno = 0;
 	    @parts = \(@parts);
 	    while (defined($part = shift(@parts))) {
 		$href = { };
 		$partfields = $href->{'fields'} = (MAILread_header($part))[0];
 		$href->{'body'} = $part;
 		$href->{'filtered'} = 0;
+		$partfields->{'x-mha-part-number'} = ++$partno;
 		$pctype = extract_ctype(
 		    $partfields->{'content-type'}, $ctype);
 
@@ -654,12 +706,16 @@ sub MAILread_body {
 		    } elsif (!defined($partfields->{'content-base'})) {
 			$partfields->{'content-base'} = [ $uribase ];
 		    }
+
+		    $partfields->{'x-mha-parent-header'} = $fields;
 		}
 	    }
 
 	    my($entity);
 	    ENTITY: foreach $entity (@entity) {
-		next  if $entity->{'filtered'};
+		if ($entity->{'filtered'}) {
+		    next ENTITY;
+		}
 
 		## If content-type not defined for part, then determine
 		## content-type based upon multipart subtype.
@@ -711,10 +767,21 @@ sub MAILread_body {
 		}
 	    }
 
+	    ## Aid garbage collection(?)
+	    foreach $entity (@entity) {
+		delete $entity->{'fields'}{'x-mha-parent-header'};
+	    }
+
 	## Else if message/rfc822 or message/news
 	} elsif ($ctype =~ m^\bmessage/(?:rfc822|news)\b^i) {
 	    $partfields = (MAILread_header($body))[0];
 
+	    # propogate parent and part no to message/* header
+	    $partfields->{'x-mha-parent-header'} =
+		$fields->{'x-mha-parent-header'};
+	    $partfields->{'x-mha-part-number'} =
+		$fields->{'x-mha-part-number'};
+
 	    $ret = &$BeginEmbeddedMesgFunc();
 	    if ($FormatHeaderFunc && defined(&$FormatHeaderFunc)) {
 		$ret .= &$FormatHeaderFunc($partfields);
@@ -729,6 +796,7 @@ sub MAILread_body {
 	    $ret .= &$EndEmbeddedMesgFunc();
 
 	    push(@files, @array);
+	    delete $partfields->{'x-mha-parent-header'};
 
 	## Else cannot handle type
 	} else {
@@ -752,13 +820,15 @@ sub MAILread_body {
 ##	($fields_hash_ref, $header_txt) = MAILread_header($mesg_data);
 ##
 sub MAILread_header {
-    my($mesg) = shift;
+    my $mesg   = shift;
 
     my $fields = { };
     my $label = '';
     my $header = '';
     my($value, $tmp, $pos);
 
+    my $encfunc = load_textencoder();
+
     ## Read a line at a time.
     for ($pos=0; $pos >= 0; ) {
 	$pos = index($$mesg, "\n");
@@ -776,10 +846,14 @@ sub MAILread_header {
 	}
 
 	## Decode text if requested
-	$tmp = &MAILdecode_1522_str($tmp,JUST_DECODE)  if $DecodeHeader;
+	if (defined($encfunc)) {
+	    $tmp = &MAILdecode_1522_str($tmp,TEXT_ENCODE);
+	} elsif ($DecodeHeader) {
+	    $tmp = &MAILdecode_1522_str($tmp,JUST_DECODE);
+	}
 
 	## Check for continuation of a field
-	if ($tmp =~ s/^\s//) {
+	if ($tmp =~ /^\s/) {
 	    $fields->{$label}[-1] .= $tmp  if $label;
 	    next;
 	}
@@ -805,7 +879,11 @@ sub MAILread_header {
 ##	($fields_hash, $header_text) = MAILread_file_header($filehandle);
 ##	
 sub MAILread_file_header {
-    my($handle) = @_;
+    my $handle = shift;
+    my $encode = shift;
+
+    my $encfunc = load_textencoder();
+
     my $label  = '';
     my $header = '';
     my $fields = { };
@@ -820,10 +898,14 @@ sub MAILread_file_header {
 	$tmp =~ s/[\r\n]//g;
 
 	## Decode text if requested
-	$tmp = &MAILdecode_1522_str($tmp,JUST_DECODE)  if $DecodeHeader;
+	if (defined($encfunc)) {
+	    $tmp = &MAILdecode_1522_str($tmp,TEXT_ENCODE);
+	} elsif ($DecodeHeader) {
+	    $tmp = &MAILdecode_1522_str($tmp,JUST_DECODE);
+	}
 
 	## Check for continuation of a field
-	if ($tmp =~ s/^\s//) {
+	if ($tmp =~ /^\s/) {
 	    $fields->{$label}[-1] .= $tmp  if $label;
 	    next;
 	}
@@ -850,6 +932,9 @@ sub MAILis_excluded {
     if ($MIMEExcs{$ctype}) {
 	return 1;
     }
+    if ($ctype =~ s/\/x-/\//) {
+	return 1  if $MIMEExcs{$ctype};
+    }
     if ($ctype =~ m|([^/]+)/|) {
 	return $MIMEExcs{$1};
     }
@@ -862,31 +947,40 @@ sub MAILis_excluded {
 ##	MAILread_header and MAILread_file_header routines.
 ##
 sub MAILhead_get_disposition {
-    my($hfields) = shift;
-    my($disp, $filename) = ('', '');
+    my $hfields = shift;
+    my $do_html = shift;
+
+    my($disp, $filename, $raw) = ('', '', '');
+    my $html_name = undef;
     local($_);
 
     if (defined($hfields->{'content-disposition'}) &&
 	    ($_ = $hfields->{'content-disposition'}->[0])) {
 	($disp)	= /^\s*([^\s;]+)/;
 	if (/filename="([^"]+)"/i) {
-	    $filename = $1;
+	    $raw = $1;
 	} elsif (/filename=(\S+)/i) {
-	    ($filename = $1) =~ s/;\s*$//g;
+	    ($raw = $1) =~ s/;\s*$//g;
 	}
     }
-    if (!$filename && defined($_ = $hfields->{'content-type'}[0])) {
+    if (!$raw && defined($_ = $hfields->{'content-type'}[0])) {
 	if (/name="([^"]+)"/i) {
-	    $filename = $1;
+	    $raw = $1;
 	} elsif (/name=(\S+)/i) {
-	    ($filename = $1) =~ s/;\s*$//g;
+	    ($raw = $1) =~ s/;\s*$//g;
 	}
     }
-    $filename = MAILdecode_1522_str($filename, DECODE_ALL);
+    $filename = MAILdecode_1522_str($raw, DECODE_ALL);
     $filename =~ s%.*[/\\:]%%;	# Remove any path component
     $filename =~ s/^\s+//;	# Remove leading whitespace
     $filename =~ s/\s+$//;	# Remove trailing whitespace
-    ($disp, $filename);
+    $filename =~ tr/\0-\40\t\n\r?:*"'<>|\177-\377/_/;
+				# Remove questionable/invalid characters
+
+    # Only provide HTML display version if requested
+    $html_name = MAILdecode_1522_str($raw)  if $do_html;
+
+    ($disp, $filename, $raw, $html_name);
 }
 
 ##---------------------------------------------------------------------------##
@@ -923,10 +1017,10 @@ sub MAILparse_parameter_str {
     my $str     = shift;        # Input string
     my $hasmain = shift;        # Flag if there is a main value to extract
 
-    require 'rfc822.pl';
+    require MHonArc::RFC822;
 
     my $parm	= { };
-    my(@toks)   = (rfc822::uncomment($str));
+    my @toks    = MHonArc::RFC822::uncomment($str);
     my($tok, $name, $value, $charset, $lang, $part);
 
     $parm->{'x-main'} = shift @toks  if $hasmain;
@@ -1085,7 +1179,7 @@ qq|<blockquote><small>---&nbsp;<i>Begin&nbsp;Message</i>&nbsp;---</small>\n|;
 ##	(ie message/rfc822 or message/news).
 ##
 sub endEmbeddedMesg {
-qq|<small>---&nbsp;<i>End Message</i>&nbsp;---</small></blockquote>\n|;
+qq|<br><small>---&nbsp;<i>End Message</i>&nbsp;---</small></blockquote>\n|;
 }
 
 ##---------------------------------------------------------------------------##
@@ -1119,6 +1213,28 @@ sub get_filter_args {
     }
     $args;
 }
+sub load_textencoder {
+    return undef  unless $TextEncode;
+    TRY: {
+	if (!defined($TextEncoderFunc)) {
+	    last TRY;
+	}
+	if (defined(&$TextEncoderFunc)) {
+	    return $TextEncoderFunc;
+	}
+	if (!defined($TextEncoderSrc)) {
+	    last TRY;
+	}
+	require $TextEncoderSrc;
+	if (defined(&$TextEncoderFunc)) {
+	    return $TextEncoderFunc;
+	}
+    }
+    warn qq/Warning: Unable to load text encode for "$TextEncode"\n/;
+    $TextEncode = undef;
+    $TextEncoderFunc = undef;
+    $TextEncoderSrc = undef;
+}
 
 ##---------------------------------------------------------------------------##
 ##	extract_ctype() extracts the content-type specification from
@@ -1141,36 +1257,124 @@ sub extract_ctype {
 }
 
 ##---------------------------------------------------------------------------##
-
+##	apply_base_url(): Convert a relative URL to a full URL with
+##	specific base;
+##
 sub apply_base_url {
-    my($b, $u) = @_;
+    my $b = shift;  # Base URL
+    my $u = shift;  # URL to apply base to
+
+    ## If no base, nothing to do
     return $u  if !defined($b) || $b !~ /\S/;
 
-    my($ret);
+    ## If absolute URL or scroll link; do nothing
     $u =~ s/^\s+//;
-    if ($u =~ m%^$Url%o || $u =~ m/^#/) {
-	## Absolute URL or scroll link; do nothing
-        $ret = $u;
-    } else {
-	## Relative URL
-	if ($u =~ /^\./) {
-	    ## "./---" or "../---": Need to remove and adjust base
-	    ## accordingly.
-	    $b =~ s/\/$//;
-	    my @a = split(/\//, $b);
-	    my $cnt = 0;
-	    while ( $cnt <= scalar(@a) &&
-		    $u =~ s|^(\.{1,2})/|| ) { ++$cnt  if length($1) == 2; }
-	    splice(@a, -$cnt)  if $cnt > 0;
-	    $b = join('/', @a, "");
-
-	} elsif ($u =~ m%^/%) {
-	    ## "/---": Just use hostname:port of base.
-	    $b =~ s%^(${Url}[^/]*)/.*%$1%o;
-	}
-        $ret = $b . $u;
+    if ($u =~ /^$UrlRxStr/o || $u =~ m/^#/) {
+        return $u;
     }
-    $ret;
+
+    ## Check if base URL allows relative resolution
+    my($host_part, $scheme);
+    if ($b =~ s{^((https?|ftp|file|nfs|acap|tftp)://[\w\-:\d.\@\%=~&]+)/?}{}) {
+	$host_part = $1;
+	$scheme    = lc $2;
+    } else {
+	warn qq/Warning: Invalid base url, "$b" to apply to "$u"\n/;
+	return $u;
+    }
+
+    ## If "/---", just use hostname:port of base.
+    if ($u =~ /^\//) {
+	return $host_part . $u;
+    }
+
+    ## Clean up base URL
+    SCHEME: {
+	if ($scheme eq 'http' || $scheme eq 'https' || $scheme eq 'acap') {
+	    $b =~ s/\?.*$//;
+	    last SCHEME;
+	}
+	if ($scheme eq 'ftp') {
+	    $b =~ s/;type=.$//;
+	    last SCHEME;
+	}
+	if ($scheme eq 'tftp') {
+	    $b =~ s/;mode=\w+$//;
+	    last SCHEME;
+	}
+    }
+    $b =~ s/\/$//;    # strip any trailing '/' (we add it back later)
+
+    ## "./---" or "../---": Need to remove and adjust base accordingly.
+    my $cnt = 0;
+    while ( $u =~ s|^(\.{1,2})/|| ) { ++$cnt  if length($1) == 2; }
+
+    if ($b eq '') {
+	# base is just host
+	return join('/', $host_part, $u);
+    }
+    if ($cnt > 0) {
+	# trim path
+	my @a = split(/\//, $b);
+	if ($cnt <= scalar(@a)) {
+	    splice(@a, -$cnt);
+	    return join('/', $host_part, @a, $u);
+	}
+	# invalid relative path, tries to go past root
+	return join('/', $host_part, $u);
+
+    }
+    return join('/', $host_part, $b, $u);
+}
+
+##---------------------------------------------------------------------------##
+
+sub extract_charset {
+    my $content = shift;  # Content-type string of entity
+    my $subtype = shift;  # Text sub-type
+    my $body    = shift;  # Reference to entity text
+    my $charset = $TextDefCharset;
+
+    if ($content =~ /\bcharset\s*=\s*([^\s;]+)/i) {
+	$charset =  lc $1;
+	$charset =~ s/['";\s]//g;
+    }
+
+    # If HTML, check <meta http-equiv=content-type> tag since it
+    # can be different than what is specified in the entity header.
+    if (($subtype eq 'html' || $subtype eq 'x-html') &&
+	($body =~ m/(<meta\s+http-equiv\s*=\s*['"]?
+		     content-type\b[^>]*>)/xi)) {
+	my $meta = $1;
+	if ($meta =~ m/\bcharset\s*=\s*['"]?([\w\.\-]+)/i) {
+	    $charset = lc $1;
+	}
+    }
+    $charset = $MIMECharsetAliases{$charset}
+	if $MIMECharsetAliases{$charset};
+
+    # If us-ascii, but 8-bit chars in body, we change to iso-8859-1
+    if ($charset eq 'us-ascii') {
+	$charset = 'iso-8859-1'  if $$body =~ /[\x80-\xFF]/;
+    }
+    $charset;
+}
+
+##---------------------------------------------------------------------------##
+##	gen_full_part_number creates a full part number of an entity
+##	from the given entity header.
+##
+sub gen_full_part_number {
+    my $fields = shift;
+    my @number = ( );
+    while (defined($fields->{'x-mha-parent-header'})) {
+	unshift(@number, ($fields->{'x-mha-part-number'} || '1'));
+	$fields = $fields->{'x-mha-parent-header'};
+    }
+    if (!scalar(@number)) {
+	return $fields->{'x-mha-part-number'} || '1';
+    }
+    join('.', @number);
 }
 
 ##---------------------------------------------------------------------------##
